pro get_atm_temp,level1b_file,tempfile,emissivity=emissivity,brightness=brightness

; Converts ATM radiances to temperature and outputs it in a copy of the ATM
; level 1B file with band 11 replaced by scaled temperature.
; Also replaces band 1 (useless anyway!) with a sinusoidal weighting function
; from zero at the two ends to 65535 in the middle of the scan.
; If emissivity is not set, a value of 0.98 typical of water is used.
; If brightness is set, the emissivity is set to 1 so brightness temperature is
; output.

scale=100.
lambda=8000.+findgen(141)*50.
response=[.355,.601,1.516,3.861,9.228,18.909,32.217,49.088,66.147,74.362, $
  73.921,74.65,79.395,85.655,89.796,89.961,86.917,83.729,81.554,81.689,82.382, $
  83.753,85.398,87.673,90.818,93.771,97.239,99.093,99.788,99.303,97.256, $
  94.367,91.656,89.055,89.012,89.984,91.436,93.076,96.083,97.547,97.839, $
  96.276,95.595,93.101,90.414,89.376,88.846,88.937,89.45,89.09,88.989,89.273, $
  89.493,91.365,91.361,90.564,93.651,96.481,94.359,89.362,83.905,79.661, $
  77.323,73.96,71.288,68.293,63.318,59.337,56.775,54.588,52.397,50.389,48.655, $
  46.431,44.638,42.024,41.344,39.604,39.065,38.557,38.788,38.903,39.965, $
  40.435,41.115,41.589,40.722,40.448,39.724,39.7,40.509,39.513,39.043,38.721, $
  38.377,37.527,36.191,35.059,35.889,32.06,30.039,27.925,25.09,23.672,20.754, $
  18.254,16.791,15.736,12.703,10.61,8.807,8.775,7.364,6.668,6.233,4.96,3.874, $
  3.444,3.832,4.096,3.517,3.584,1.962,2.043,2.2,2.027,2.063,1.373,2.123,1.357, $
  1.182,1.296,.652,1.234,.645,.676,.695,.687,.577,.686,.757]
if n_elements(emissivity) gt 0 then eps=emissivity[0] else $
  if keyword_set(brightness) then eps=1. else $
  eps=.98 ; water emissivity, could make it angle dependent
if ~hdf_ishdf(level1b_file) then begin
  ; let's try an ENVI BIP - look for the header
  if ~file_test(level1b_file+'.hdr') then $
    message,'Unidentified level 1B file - not HDF or BIP'
  openr,unit,level1b_file+'.hdr',/get
  str=''
  repeat readf,unit,str until strcmp(strmid(str,0,7),'samples',/fold)
  ns=long(strmid(str,strpos(str,'=')+1))
  repeat readf,unit,str until strcmp(strmid(str,0,5),'lines',/fold)
  nl=long(strmid(str,strpos(str,'=')+1))
  repeat readf,unit,str until strcmp(strmid(str,0,5),'bands',/fold)
  nb=long(strmid(str,strpos(str,'=')+1))
  if ~max(nb eq [1,11]) then message,'Need 1 or 11 bands in ENVI file'
  repeat readf,unit,str until strcmp(strmid(str,0,10),'interleave',/fold)
  format=strlowcase(strtrim(strmid(str,strpos(str,'=')+1),2))
  if format ne 'bip' then $
    message,'Level 1B file format '+format+' not currently supported'
  bip=1
  close,unit
  openr,unit,level1b_file
  if nb eq 11 then indata=uintarr(11,ns) else dn=uintarr(ns)
  if ~file_test(tempfile) then begin
    temp_sdid=hdf_sd_start(tempfile,/create)
    temp_sdsid=hdf_sd_create(temp_sdid,'ATdata',[ns,nl,11],/dfnt_uint16)
    hdf_sd_endaccess,temp_sdsid
    hdf_sd_end,temp_sdid
  endif
endif else begin
  bip=0
  if ~file_test(tempfile) then file_copy,level1b_file,tempfile
  level1b_fid=hdf_open(level1b_file)
  level1b_sdid=hdf_sd_start(level1b_file)
  index=hdf_sd_nametoindex(level1b_sdid,'ATdata')
  level1b_sdsid=hdf_sd_select(level1b_sdid,index)
  hdf_sd_getinfo,level1b_sdsid,dims=dims
  ns=dims[0]
  nl=dims[1]
endelse
file_chmod,tempfile,/u_write
weights=uint(sin(dindgen(ns)*!dpi/(ns-1))*65535d0+.5d0)
temp_sdid=hdf_sd_start(tempfile,/rdwr)
index=hdf_sd_nametoindex(temp_sdid,'ATdata')
temp_sdsid=hdf_sd_select(temp_sdid,index)
for line=0,nl-1 do begin
  if bip then begin
    if nb eq 11 then begin
      readu,unit,indata
      dn=reform(indata[10,*])
    endif else readu,unit,dn
  endif else hdf_sd_getdata,level1b_sdsid,dn,start=[0,line,10],count=[ns,1,1]
  radiance=dn*.001
  temp=t_bb(radiance,lambda,response,emissivity=eps,/trunc,t_bb_data=t_bb_data)
  dn[*]=temp*scale>0<65535
  hdf_sd_adddata,temp_sdsid,dn,start=[0,line,10],count=[ns,1,1]
  hdf_sd_adddata,temp_sdsid,weights,start=[0,line,0],count=[ns,1,1]
  check=check_math()
  if (check and not 32) ne 0 then stop
endfor
hdf_sd_endaccess,temp_sdsid
hdf_sd_end,temp_sdid
if bip then free_lun,unit else begin
  hdf_sd_endaccess,level1b_sdsid
  hdf_sd_end,level1b_sdid
  hdf_close,level1b_fid
endelse

end

